<?php

namespace drp\AdminPanel\Admin\Commands;

use Illuminate\Console\Command;
use Illuminate\Filesystem\Filesystem;

class SetupCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'admin:setup';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Sets up the admin package.';

    /**
     * @var Filesystem
     */
    protected $files;

    /**
     * This is used to decide whether to migrate and seed the user table.
     *
     * @var boolean
     */
    protected $seedUser = true;

    public function __construct(Filesystem $files)
    {
        $this->files = $files;
        parent::__construct();
    }


    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $overwriteConfig = $this->confirm('Would you like drpadmin config copied? This will overwrite any existing config file.', 'yes');
        if ($overwriteConfig) {
            $this->copyOverConfig();
            $this->info('Copied over custom drpadmin config file.');
        }

        $setupNpm = $this->confirm('Would you like to add the npm dependencies to your package.json? This includes gulp-less, gulp-minify-css, gulp-rename, gulp-concat and gulp-uglify?', 'yes');
        if ($setupNpm) {
            $this->setupNpm();
        }

        $setupGulpFile = $this->confirm('Would you like to add the elixir mix to your gulpfile?', 'yes');
        if ($setupGulpFile) {
            $this->setupGulpFile();
        }

        $publishAssets = $this->confirm('Would you like to publish the admin assets?', 'yes');
        if ($publishAssets) {
            $this->publishAssets();
        }

        $migrateAndSeed = $this->confirm('Would you like to run migrate and seed? This will seed an example user into the DB. Make sure you\'ve updated your .env to your local settings.', 'yes');
        if ($migrateAndSeed) {
            exec('composer dumpautoload -o');
            $this->call('migrate', ['--seed' => true]);

            $this->info('To log into the admin panel, use username: "admin@example.com" and password: "admin@example.com".');
        }

        $this->info('Running npm install and gulp');
        exec('npm install && gulp');

        $this->info('Finishing setting up the admin panel.');
    }

    /**
     * Runs php artisan vendor:publish
     */
    protected function publishAssets()
    {
        $exitStatus = 0;
        $output = [];
        exec('php artisan vendor:publish --provider=\'drp\AdminPanel\AdminPanelServiceProvider\'', $output, $exitStatus);
        if ($exitStatus > 0) {
            $this->error('Error in publishing gulp tasks and resources.');
        }
        $exitStatus = 0;
        if ($this->seedUser) {
            exec('php artisan vendor:publish --provider=\'drp\AdminPanel\User\UserServiceProvider\'', $output, $exitStatus);
            if ($exitStatus > 0) {
                $this->error('Error in publishing user migrations and seeds.');
            } else {
                $this->updateDatabaseSeeder();
            }
        }
        $this->info('Finishing publishing assets.');
    }

    /**
     * Updates the database seeder by calling the admin user seeder.
     */
    protected function updateDatabaseSeeder()
    {
        $databaseSeeder = $this->files->get(database_path('seeds/DatabaseSeeder.php'));
        $version = $this->laravel->version();
        $newSeederClass = '\'AdminUsersSeeder\'';
        if (version_compare($version, '5.1.0', '>=')) {
            $newSeederClass = 'AdminUsersSeeder::class';
        }

        if (strpos($databaseSeeder, $newSeederClass) === false) {
            $databaseSeeder = str_replace(
                'public function run()
    {',
                'public function run()
    {
        $this->call(' . $newSeederClass . ');',
                $databaseSeeder
            );
        }

        $this->files->put(database_path('seeds/DatabaseSeeder.php'), $databaseSeeder);

        $this->info('Updated database seeder to call admin user seeder.');
    }


    /**
     * Adds the require and mix.admin() to the local gulpfile.js
     */
    protected function setupGulpFile()
    {
        $gulpFile = $this->files->get(base_path('gulpfile.js'));

        if (strpos($gulpFile, 'require(\'./gulp-tasks/admin-elixir\');') === false) {
            $gulpFile = str_replace(
                'var elixir = require(\'laravel-elixir\');',
                'var elixir = require(\'laravel-elixir\');
require(\'./gulp-tasks/admin-elixir\');',
                $gulpFile
            );
        }

        if (strpos($gulpFile, 'mix.admin();') === false) {
            $gulpFile = str_replace(
                'elixir(function(mix) {',
                'elixir(function(mix) {
    mix.admin();',
                $gulpFile
            );
        }
        $this->files->put(base_path('gulpfile.js'), $gulpFile);
        $this->info('Updated gulpfile.js.');
    }


    /**
     * Adds the gulp dependencies to the local package.json.
     */
    protected function setupNpm()
    {
        $newDependencies = [
            'gulp-concat'     => '^2.6.0',
            'gulp-less'       => '^3.0.5',
            'gulp-minify-css' => '^1.2.1',
            'gulp-rename'     => '^1.2.2',
            'gulp-uglify'     => '^1.5.1',
        ];

        $packageJSON = $this->files->get(base_path('package.json'));
        $package = json_decode($packageJSON, true);
        if (!$package) {
            $this->error('Could not read the package.json, please check the syntax.');
            return;
        }

        $dependencies = $package['dependencies'];
        $existingDependencies = [];
        foreach ($newDependencies as $dependency => $version) {
            if (isset($dependencies[$dependency])) {
                $existingDependencies[] = $dependency;
            } else {
                $dependencies[$dependency] = $version;
            }
        }
        if (count($existingDependencies) > 0) {
            $this->warn('Not installing ' . implode(', ', $existingDependencies) . ' as they are already in the package.json');
        }
        $package['dependencies'] = $dependencies;
        $this->files->put(base_path('package.json'), json_encode($package, JSON_PRETTY_PRINT));
        $this->info('Updated the package json');
    }

    /**
     * Copies over the drpadmin config and updates the auth config if the user
     * requests it.
     */
    public function copyOverConfig()
    {
        $configContents = $this->files->get(__DIR__.'/../../../config/drpadmin.php');
        $configDestination = config_path('drpadmin.php');

        $enableUserCRUD = $this->confirm('Would you like to enable user CRUD?', 'yes');

        if (!$enableUserCRUD) {
            $configContents = $this->disableUserCRUD($configContents);
        }

        $this->files->put($configDestination, $configContents);

        $setupAuth = $this->confirm('Would you like the auth config updated, so that it works with this package?', 'yes');
        if ($setupAuth) {
            $this->setupAuthConfig();
        }
    }

    /**
     * Updates the auth config to work with the admin package.
     */
    protected function setupAuthConfig()
    {
        $authConfig = $this->files->get(config_path('auth.php'));
        $version = $this->laravel->version();

        $oldModelClass = '\'App\User\'';
        $newModelClass = '\'drp\AdminPanel\User\Repository\AdminUser\'';
        if (version_compare($version, '5.1.0', '>=')) {
            $oldModelClass = 'App\User::class';
            $newModelClass = 'drp\AdminPanel\User\Repository\AdminUser::class';
        }

        $authConfig = str_replace(
            '\'model\' => ' . $oldModelClass,
            '\'model\' => ' .  $newModelClass,
            $authConfig
        );

        $authConfig = str_replace(
            '\'table\' => \'users\'',
            '\'table\' => \'admin_users\'',
            $authConfig
        );

        $authConfig = str_replace(
            '\'email\' => \'emails.password\'',
            '\'email\' => \'DrpAuthAdmin::emails.password\'',
            $authConfig
        );

        $this->files->put(config_path('auth.php'), $authConfig);
        $this->info('Updated auth config file.');
    }


    /**
     * Sets the includeAdminUserCrud in the config to false.
     */
    protected function disableUserCRUD($configContents)
    {
        $this->seedUser = false;

        $configContents = str_replace(
            '\'includeAdminUserCrud\' => true',
            '\'includeAdminUserCrud\' => false',
            $configContents
        );
        return $configContents;
    }
}
